
// this code is auto-generated by the script 'pyclesperanto_autogen_tier_script.ipynb'.
// Do not edit manually. Instead, edit the script and run it again.
    
#include "pycle_wrapper.hpp"
#include "tier3.hpp"

namespace py = pybind11;

auto tier3_(py::module &m) -> void {

    
m.def("_exclude_labels_on_edges", &cle::tier3::exclude_labels_on_edges_func, "Call exclude_labels_on_edges from C++.",
    py::return_value_policy::take_ownership,
    py::arg("device"), py::arg("src"), py::arg("dst"), py::arg("exclude_x"), py::arg("exclude_y"), py::arg("exclude_z"));

    
m.def("_flag_existing_labels", &cle::tier3::flag_existing_labels_func, "Call flag_existing_labels from C++.",
    py::return_value_policy::take_ownership,
    py::arg("device"), py::arg("src"), py::arg("dst"));

    
m.def("_gamma_correction", &cle::tier3::gamma_correction_func, "Call gamma_correction from C++.",
    py::return_value_policy::take_ownership,
    py::arg("device"), py::arg("src"), py::arg("dst"), py::arg("gamma"));

    
m.def("_histogram", &cle::tier3::histogram_func, "Call histogram from C++.",
    py::return_value_policy::take_ownership,
    py::arg("device"), py::arg("src"), py::arg("dst"), py::arg("nbins"), py::arg("min"), py::arg("max"));

    
m.def("_mean_of_all_pixels", &cle::tier3::mean_of_all_pixels_func, "Call mean_of_all_pixels from C++.",
    py::return_value_policy::take_ownership,
    py::arg("device"), py::arg("src"));


}
