
// this code is auto-generated by the script 'pyclesperanto_autogen_tier_script.ipynb'.
// Do not edit manually. Instead, edit the script and run it again.
    
#include "pycle_wrapper.hpp"
#include "tier7.hpp"

namespace py = pybind11;

auto tier7_(py::module &m) -> void {

    
m.def("_voronoi_otsu_labeling", &cle::tier7::voronoi_otsu_labeling_func, "Call voronoi_otsu_labeling from C++.",
    py::return_value_policy::take_ownership,
    py::arg("device"), py::arg("src"), py::arg("dst"), py::arg("spot_sigma"), py::arg("outline_sigma"));


}
