
# this code is auto-generated by the script 'pyclesperanto_autogen_tier_script.ipynb'.
# Do not edit manually. Instead, edit the script and run it again.

from ._core import Device
from ._array import Image
from ._decorators import plugin_function


@plugin_function
def masked_voronoi_labeling(
    input_image: Image,
	mask: Image = None,
	output_image: Image = None,
	device: Device = None
) -> Image:
    from ._pyclesperanto import _masked_voronoi_labeling as op

    return op(
        device=device,
		src=input_image,
		mask=mask,
		dst=output_image
    )


@plugin_function
def voronoi_labeling(
    input_image: Image,
	output_image: Image = None,
	device: Device = None
) -> Image:
    from ._pyclesperanto import _voronoi_labeling as op

    return op(
        device=device,
		src=input_image,
		dst=output_image
    )

