# pyclesperanto version
VERSION_CODE = (0, 7, 0)
VERSION_STATUS = ""
VERSION = ".".join(str(x) for x in VERSION_CODE) + VERSION_STATUS

# clic version
CLIC_VERSION_CODE = (0, 7, 0)
CLIC_VERSION_STATUS = ""
CLIC_VERSION = ".".join(str(x) for x in CLIC_VERSION_CODE) + CLIC_VERSION_STATUS

COMMON_ALIAS = "cle"
